/// <reference types="node" />
import { Environment } from '@mockoon/commons';
import { RequestListener } from 'http';
import ServerlessHttp from 'serverless-http';
export declare class MockoonServerless {
    private environment;
    private options;
    constructor(environment: Environment, options?: {
        logTransaction: boolean;
        /**
         * List of routes uuids to disable.
         * Can also accept strings containing a route partial path, e.g. 'users' will disable all routes containing 'users' in their path.
         */
        disabledRoutes: string[];
    });
    /**
     * Returns an Express request listener that can be used in serverless environments
     * and with the serverless-http library.
     *
     * @returns
     */
    requestListener(): RequestListener;
    /**
     * Returns a serverless-http wrapped request listener for AWS Lambda.
     *
     * @returns
     */
    awsHandler(): ServerlessHttp.Handler;
    /**
     * Returns a request listener for Google Cloud Functions.
     *
     * @returns
     */
    firebaseApp(): RequestListener;
    /**
     * Returns a serverless-http wrapped request listener for Netlify.
     *
     * @returns
     */
    netlifyHandler(): RequestListener;
}
