import { createLoggerInstance, listenServerEvents, MockoonServer } from '@mockoon/commons-server';
import ServerlessHttp from 'serverless-http';
export class MockoonServerless {
    constructor(environment, options = {
        logTransaction: false,
        disabledRoutes: []
    }) {
        this.environment = environment;
        this.options = options;
        if (!environment) {
            throw new Error('No environment data provided');
        }
    }
    /**
     * Returns an Express request listener that can be used in serverless environments
     * and with the serverless-http library.
     *
     * @returns
     */
    requestListener() {
        const logger = createLoggerInstance();
        const server = new MockoonServer(this.environment, {
            disabledRoutes: this.options.disabledRoutes
        });
        listenServerEvents(server, this.environment, logger, this.options.logTransaction);
        return server.createRequestListener();
    }
    /**
     * Returns a serverless-http wrapped request listener for AWS Lambda.
     *
     * @returns
     */
    awsHandler() {
        return ServerlessHttp(this.requestListener());
    }
    /**
     * Returns a request listener for Google Cloud Functions.
     *
     * @returns
     */
    firebaseApp() {
        return this.requestListener();
    }
    /**
     * Returns a serverless-http wrapped request listener for Netlify.
     *
     * @returns
     */
    netlifyHandler() {
        return ServerlessHttp(this.requestListener());
    }
}
//# sourceMappingURL=serverless.js.map